/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer: Benoit Pelletier
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.ant.cluster;

/**
 * Define ScriptCluster task
 * @author Benoit Pelletier
 */
public class ScriptCluster extends ClusterTasks {

    /**
     * Info for the logger
     */
    private static final String INFO = "[ScriptCluster] ";

    /**
     * domain name
     */
    private String domainName = null;

    /**
     * script dir
     */
    private String scriptDir = null;

    /**
     * nodes name
     */
    private String clusterNodesName = null;

    /**
     * Default constructor
     */
    public ScriptCluster() {
        super();
    }

    /**
     * Set domainName
     * @param domainName domain name
     */
    public void setDomainName(String domainName) {
        this.domainName = domainName;
    }

    /**
     * Set script directory
     * @param scriptDir Script directory
     */
    public void setScriptDir(String scriptDir) {
        this.scriptDir = scriptDir;
    }

    /**
     * set the name prefix for the cluster nodes
     * @param clusterNodesName prefix of the nodes names in the cluster
     */
    public void setClusterNodesName(String clusterNodesName) {
        this.clusterNodesName = clusterNodesName;
    }

    /**
     * Generates the carol tasks for each JOnAS's instances
     */
    public void generatesTasks() {

        int portInd = 0;

        // creation of the Script tasks
        Script script = new Script();
        script.setDomainName(domainName);
        script.createFiles(scriptDir);

        for (int i = getDestDirSuffixIndFirst(); i <= getDestDirSuffixIndLast(); i++) {

            String destDir = getDestDir(getDestDirPrefix(), i);
            log(INFO + "tasks generation for " + destDir);

            script.addNode(clusterNodesName, destDir);


            portInd++;
        }
        script.flushFiles();
        addTasks(script);
    }
}