/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 * Copyright (C) 2006 Distributed Systems Lab.
 * Universidad Polit??cnica de Madrid (Spain)
 * Contact: http://lsd.ls.fi.upm.es/lsd
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.ha;

import java.io.ByteArrayInputStream;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;

import javax.ejb.SessionBean;

import org.objectweb.carol.cmi.ha.HaUtilsDelegate;
import org.objectweb.carol.cmi.ha.ReplicationManager;
import org.objectweb.jonas.service.ServiceManager;
import org.objectweb.jonas_ejb.container.JStatefulContext;
import org.objectweb.jonas_ejb.container.JStatefulInputStream;
import org.objectweb.jonas_ejb.container.JStatefulSwitch;

/**
 * This class contains useful methods related to the HA service
 * @author Francisco Perez-Sorrosal (fpsorrosal@no-spam@fi.upm.es)
 * @author Alberto Paz-Jimenez (apaz@no-spam@fi.upm.es)
 * @author benoit pelletier
 */
public class HaUtilsDelegateImpl implements HaUtilsDelegate {

    /**
     * Replication Manager
     */
    private static ReplicationManager rm = null;

    /**
     * Constructor
     *
     */
    public HaUtilsDelegateImpl() {
    }

    /**
     * @return get the replication service
     */
    public static ReplicationManager getReplicationManager() {
        if (rm == null) {
            HaService svc = null;
            try {
                // Access the high-availability service (HA)
                svc = (HaService) ServiceManager.getInstance().getService("ha");
            } catch (Exception e) {
                e.printStackTrace();
            }
            // Get the replication manager
            rm = svc.getReplicationManager();
        }
        return rm;
    }

    /**
     * Injects the serialized state of the bean in the corresponding beanSwitch
     * @param bean target bean
     * @param beanState state to inject
     * @throws Exception if an error occurs
     */
    public void injectState(Object bean, byte[] beanState) throws Exception {

        JStatefulSwitch beanSwitch = null;

        if (bean instanceof JStatefulSwitch) {
            beanSwitch = (JStatefulSwitch) bean;
        } else {
            // never happen
            throw new Exception("Unexpected bean");
        }
        JStatefulContext bctx = (JStatefulContext) beanSwitch.getStatefulContext();
        try {
            ByteArrayInputStream bais = new ByteArrayInputStream(beanState);
            JStatefulInputStream in = new JStatefulInputStream(bais, beanSwitch);
            Object obj = in.readObject();
            //if (TraceCmi.isDebugCmiHA())
            //  TraceCmi.debugCmiHA("\tDeserialized object:" + obj);
            in.close();
            SessionBean sb = (SessionBean) obj;
            
            // Deal with transient values
            SessionBean oldInstance = bctx.getInstance();
            Field[] fields = sb.getClass().getDeclaredFields();
            for (int i = 0; i < fields.length; i++) {
                if (Modifier.isTransient(fields[i].getModifiers())) {
                    try {
                        fields[i].setAccessible(true);
                        fields[i].set(sb,fields[i].get(oldInstance));
                    } catch (IllegalAccessException iae) {
                        iae.printStackTrace();
                    } catch (SecurityException se) {
                        se.printStackTrace();
                    }
                }
            }
            
            bctx.setInstance(sb);
            sb.ejbActivate();
            //TraceCmi.debugCmiHA("\tDeserialization is completed!!!");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
