/**
* High Availability Service (HA) for JOnAS
*
* Copyright (C) 2006 Distributed Systems Lab.
* Universidad Politécnica de Madrid (Spain)
* Contact: http://lsd.ls.fi.upm.es/lsd
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id$
* --------------------------------------------------------------------------
*/

package org.objectweb.jonas.ha.interceptor;

import java.util.Hashtable;
import java.util.Iterator;

import org.objectweb.carol.cmi.ha.Constants;
import org.objectweb.carol.rmi.jrmp.interceptor.JServiceContext;

/**
 * This class holds the required information to pass between client/server
 * through HA interceptors
 * @author Francisco Perez-Sorrosal (fpsorrosal@no-spam@fi.upm.es)
 * @author Alberto Paz-Jimenez (apaz@no-spam@fi.upm.es)
 */
public class HAContext  implements JServiceContext {

	/**
     * Holds the required data to transfer as a pair string/object
     */
    private Hashtable data = new Hashtable();

    /**
     * Constructor
     */
    public HAContext() {
    }

    /**
     * Returns the HA context identifier
     * @return the HA context id
     */
    public int getContextId() {
        return Constants.HA_CTX_ID;
    }

    /**
     * Sets concrete data to pass between client and server
     * @param param the key value
     * @param value the data associated to the key value
     */
    public void setData(String param, Object value) {
        data.put(param, value);
    }

    /**
     * Returns the data identified by the key param
     * @param param the key value to get the data from
     * @return the required data
     */
    public Object getData(String param) {
        return data.get(param);
    }

    /**
     * Returns a string representation of the object
     * @return the string representing the HAContext
     */
    public String toString() {
		String ret = "\nHAContext: " + Constants.HA_CTX_ID + "\n";

		ret += "\tParams:\n";
        Iterator it = data.keySet().iterator();
        while (it.hasNext()) {
            String key = (String) it.next();
            ret += "\t\t" + key + "\t" + data.get(key) + "\n";
        }

		return ret;

    }
}