/**
 * High Availability Service (HA) for JOnAS
 *
 * Copyright (C) 2006 Distributed Systems Lab.
 * Universidad Politécnica de Madrid (Spain)
 * Contact: http://lsd.ls.fi.upm.es/lsd
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.ha.interceptor;

import org.objectweb.carol.cmi.ha.RequestId;

/**
 * This is the class stored in the Thread local variable of HACurrent class and
 * represents the thread associated HA context
 * @author Francisco Perez-Sorrosal (fpsorrosal@no-spam@fi.upm.es)
 * @author Alberto Paz-Jimenez (apaz@no-spam@fi.upm.es)
 */
public class HACurrentContext {

    /**
     * Holds info about the current call
     */
    private RequestId currentReq = null;

    /**
     * Holds info about the next call to be performed
     */
    private RequestId nextReq = null;

    /**
     * Holds the info about the root request
     */
    private RequestId rootReq = null;

    /**
     * Identifies if replication is needed
     */
    private boolean replicate = false;

    /**
     * Constructor
     */
    public HACurrentContext() {
    }

    /**
     * Get the current request
     * @return the current request
     */
    public RequestId getCurrentReq() {
        return currentReq;
    }

    /**
     * Set the current request
     * @currentReq the current request
     */
    public void setCurrentReq(RequestId currentReq) {
        this.currentReq = currentReq;
    }

    /**
     * Get the next request
     * @return the next request
     */
    public RequestId getNextReq() {
        return nextReq;
    }

    /**
     * Set the next request
     * @nextReq the next request
     */
    public void setNextReq(RequestId nextReq) {
        this.nextReq = nextReq;
    }

    /**
     * Get the root request
     * @return the root request
     */
    public RequestId getRootReq() {
        return rootReq;
    }

    /**
     * Set the root request
     * @param rootReq the root request
     */
    public void setRootReq(RequestId rootReq) {
        this.rootReq = rootReq;
    }

    /**
     * Get if it is necessary to replicate
     * @return true if required
     */
    public boolean isReplicate() {
        return replicate;
    }

    /**
     * Set if it is necessary to replicate
     * @param replicate
     */
    public void setReplicate(boolean replicate) {
        this.replicate = replicate;
    }

}
