/**
* High Availability Service (HA) for JOnAS
*
* Copyright (C) 2006 Distributed Systems Lab.
* Universidad Politécnica de Madrid (Spain)
* Contact: http://lsd.ls.fi.upm.es/lsd
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id$
* --------------------------------------------------------------------------
*/

package org.objectweb.jonas.ha.interceptor;

import java.io.IOException;

import org.objectweb.carol.cmi.configuration.TraceCmi;
import org.objectweb.carol.cmi.ha.Constants;
import org.objectweb.carol.cmi.ha.RequestId;
import org.objectweb.carol.rmi.jrmp.interceptor.JServerRequestInfo;
import org.objectweb.carol.rmi.jrmp.interceptor.JServerRequestInterceptor;
import org.omg.PortableInterceptor.ForwardRequest;
import org.omg.PortableInterceptor.ServerRequestInfo;

/**
 * This class represents the server side of the interceptor mechanism
 * implemented to transfer replication related information between client
 * and server sides
 * @author Francisco Perez-Sorrosal (fpsorrosal@no-spam@fi.upm.es)
 * @author Alberto Paz-Jimenez (apaz@no-spam@fi.upm.es)
 */
public class HAServerInterceptor implements JServerRequestInterceptor {

    /**
     * Interceptor name
     */
    private String interceptorName = "HAServerInterceptor";

    /**
     * Constructor
     */
    public HAServerInterceptor() {
    }

    /**
     * Get the name of this interceptor
     * @return name
     */
    public String name() {
        return interceptorName;
    }

    /**
     * Analyzes the HAContext received and constructs the required
     * context on the HA current context (the thread associated class) on
     * the server side.
     */
    public void receive_request(JServerRequestInfo jri) throws IOException {
    	TraceCmi.debugCmiHA("----------------------------------");
    	TraceCmi.debugCmiHA("HAServerInterceptor: receive_request");
    	// Get the HAContext from the received info
        HAContext receivedCtx = (HAContext) jri.get_request_service_context(Constants.HA_CTX_ID);
       	if (TraceCmi.isDebugCmiHA())
       		TraceCmi.debugCmiHA(receivedCtx.toString());
        // Get the current Thread/HAContext association class
        HACurrentDelegateImpl current = HACurrentDelegateImpl.getCurrent();
        RequestId req = (RequestId) receivedCtx.getData("req_id");
        current.setCurrentReq(req);
        // If a root_ctx has not been propagated, the call refers to the
        // top-level EJB, then set this reqId as the root_ctx and remind
        // that we have to replicate when returning.
        // Otherwise save the root_ctx it in the thread associated variable.
        if(receivedCtx.getData("root_ctx") == null) {
        	current.setRootReq(req);
        	TraceCmi.debugCmiHA("RootReq: " + req);
        	current.setReplicate(true);
        } else {
        	RequestId root = (RequestId) receivedCtx.getData("root_ctx");
        	current.setRootReq(root);
        	TraceCmi.debugCmiHA("RootReq: " + root);
        	current.setReplicate(false);
        }
   		TraceCmi.debugCmiHA("HACtx asocciated to Thread");
  		TraceCmi.debugCmiHA("----------------------------------");
    }

    //-------------------------------------------------------------------------
    // Other unused methods
    //-------------------------------------------------------------------------

    public void send_reply(JServerRequestInfo jri) throws IOException {
    }

    public void destroy() {

    }

    public void receive_request_service_contexts(ServerRequestInfo ri)
            throws ForwardRequest {
    }

    public void send_exception(JServerRequestInfo jri) throws IOException {
    }

    public void send_other(JServerRequestInfo jri) throws IOException {
    }
}
