/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.deploy;

import java.io.IOException;
import java.util.ArrayList;

import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.objectweb.jonas.jmx.J2eeObjectName;
import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.webapp.jonasadmin.Jlists;
import org.objectweb.jonas.webapp.jonasadmin.JonasAdminJmx;

/**
 *
 * @author Patrick Smith
 *         Greg Lapouchnian
 *         Adriana Danes
 */

public class EditDomainDeployAction extends BaseDeployAction {

    static final int indentUnit = 2;
    static final char indentChar = '_';
// --------------------------------------------------------- Public Methods

    public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
        , HttpServletRequest p_Request, HttpServletResponse p_Response)
        throws IOException, ServletException {

        // Form used
        DomainDeployForm oForm = (DomainDeployForm) p_Form;
        try {
            // Store type deployment if exists
            setCurrentJonasDeployment(p_Request);
            // Domain ObjectName
            ObjectName on = JonasAdminJmx.getJ2eeDomainObjectName();

            // All targets in the domain: started servers and clusters
            ArrayList alTargets = new ArrayList();
            ArrayList alTargetNames = new ArrayList();
            getTargetServerList(alTargetNames, alTargets, on, 0);
            // Deployable files (All files on the local server)
            ArrayList alDeployable = getListDeployableFiles();

            // Deploy files
            ArrayList alDeploy = new ArrayList(alDeployable);

            // Set in form
            oForm.setListDeployable(alDeployable);
            oForm.setListTargets(alTargets);
            oForm.setListTargetNames(alTargetNames);
            oForm.setListDeploy(alDeploy);
            oForm.setDeploy(Jlists.getString(alDeploy, Jlists.SEPARATOR));
            oForm.setIsConfigurable(isConfigurable());

        }
        catch (Throwable t) {
            addGlobalError(t);
            saveErrors(p_Request, m_Errors);
            return (p_Mapping.findForward("Global Error"));
        }
        // Forward to the jsp.
        return (p_Mapping.findForward(getForwardEdit()));
    }

    /**
     *
     * @param targetNames List of target names to be updated
     * @param targetONs List of target ONss to be updated
     * @param domainOn domain/cluster ObjectName
     * @param indent nb of indentation characters
     * @return total number of target servers
     * @throws MalformedObjectNameException
     */
    int getTargetServerList(ArrayList targetNames, ArrayList targetONs, ObjectName domainOn, int indent) throws MalformedObjectNameException {
        boolean clustNameToBeAdded = false;
        int nbServers = 0;
        String opName = "getStartedServers";
        String[] startedServers = (String[]) JonasManagementRepr.invoke(domainOn, opName, null, null);

        nbServers = startedServers.length;
        if (indent > 0) {
            // domainOn is a J2EEDomain ObjectName corresponding to a cluster
            if (nbServers > 0) {
                // insert cluster name as target shifted to left
                String targetName = domainOn.getKeyProperty("clusterName");
                targetNames.add(printableName(targetName, indent - indentUnit, indentChar));
                targetONs.add(domainOn.toString());
            } else {
                clustNameToBeAdded = true;
            }
        }
        for (int i = 0; i < nbServers; i++) {
            String serverName = startedServers[i];
            targetNames.add(printableName(serverName, indent, indentChar));
            targetONs.add(J2eeObjectName.J2EEServer(domainOn.getDomain(), serverName).toString());
        }

        String[] clusters = (String []) JonasManagementRepr.getAttribute(domainOn, "clusters");
        for (int i = 0; i < clusters.length; i++) {
            ArrayList embededTargetNames = new ArrayList();
            ArrayList embededTargetONs = new ArrayList();
            ObjectName clusterOn = ObjectName.getInstance(clusters[i]);
            int nbEmbededServers = getTargetServerList(embededTargetNames, embededTargetONs, clusterOn, indent + indentUnit);
            if (clustNameToBeAdded && nbEmbededServers > 0) {
                // insert cluster name as target if not already dome
                String targetName = domainOn.getKeyProperty("clusterName");
                targetNames.add(printableName(targetName, indent - indentUnit, indentChar));
                targetONs.add(domainOn.toString());
                clustNameToBeAdded = false;
            }
            // add inner targets
            targetNames.addAll(embededTargetNames);
            targetONs.addAll(embededTargetONs);
            nbServers = nbServers + nbEmbededServers;
        }

        return nbServers;
    }

    private String printableName(String name, int indent, char indentChar) {
        StringBuffer buf = new StringBuffer();
        for (int j = 0; j < indent; j++) {
            buf.insert(j, indentChar);
        }
        buf.append(name);
        return new String(buf);
    }
}
