/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2003-2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.domain;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.objectweb.jonas.jmx.J2eeObjectName;
import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.webapp.jonasadmin.JonasBaseAction;

/**
 * @author Adriana Danes
 */
public class ApplyJonasServersConfirmMoveAction extends JonasBaseAction {

//	--------------------------------------------------------- Public Methods

    public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
            , HttpServletRequest p_Request, HttpServletResponse p_Response)
    throws IOException, ServletException {

        ArrayList alSelected = (ArrayList) m_Session.getAttribute("listServersSelected");

        // Form used
        ItemsServersForm oForm = (ItemsServersForm) p_Form;

        // Actions
        try {
            String domainName = m_WhereAreYou.getCurrentDomainName();
            ObjectName currentOn =  ObjectName.getInstance((String) m_Session.getAttribute("currentOn"));
            String clusterName = oForm.getCluster();
            ObjectName targetClusterOn = null;
            if (domainName.equals(clusterName)) {
                // The target is the domain (move from a cluster to the domain)
                targetClusterOn = ObjectName.getInstance(J2eeObjectName.J2EEDomain(domainName));
            } else {
                // The target is a cluster not the domain
                // which is the parent of the target ?
            	// it depends, if the target is the parent of the current or its sub-cluster
            	String currentClusterName = currentOn.getKeyProperty("clusterName");
                String currentParentClusterName = currentOn.getKeyProperty("parentClusterName");
                if (currentClusterName == null) {
                	// Move from the domain to a cluster (the parent is the domain)
                	targetClusterOn = ObjectName.getInstance(J2eeObjectName.J2EECluster(domainName, clusterName));
                } else {
                	String parentName = null;
                	if (currentParentClusterName.equals(clusterName)) {
                		// Move from a cluster to its parent cluster
                		// Can't determine the parent of the target, its the parent of the parent
                		ObjectName clusterOns = J2eeObjectName.J2EEClusters(domainName, clusterName);
                		Iterator it = JonasManagementRepr.queryNames(clusterOns).iterator();
                		if (it.hasNext()) {
                			targetClusterOn = (ObjectName) it.next();
                		} else {
                			throw new Throwable("Can't determine target cluster");
                		}
                	} else {
                		// Move from a cluster to its sub-cluster
                		parentName = currentClusterName;
                		targetClusterOn = ObjectName.getInstance(J2eeObjectName.J2EECluster(domainName, clusterName, parentName));
                	}
                }
            }
//			Remove servers
            for (int i = 0; i < alSelected.size(); i++) {
                JonasServerItem item = (JonasServerItem) alSelected.get(i);
                String serverOn = (String) item.getObjectName();
                String serverName = ObjectName.getInstance(serverOn).getKeyProperty("name");
                String[] signature = { "java.lang.String" };
                String[] params = { serverName };
                String[] urls =  (String[]) JonasManagementRepr.invoke(currentOn, "getConnectorServerURLs", params, signature);
                params[0] = serverOn;
                JonasManagementRepr.invoke(currentOn, "removeServer", params, signature);

                String[] signature1 = new String[2];
                Object[] params1 = new Object[2];
                signature1[0] = "java.lang.String";
                signature1[1] = "[Ljava.lang.String;";
                params1[0] = serverOn;
                params1[1] = urls;
                JonasManagementRepr.invoke(targetClusterOn, "addServer", params1, signature1);
            }

        }
        catch (Throwable t) {
            addGlobalError(t);
            saveErrors(p_Request, m_Errors);
            return (p_Mapping.findForward("Global Error"));
        }
        // Forward to the domain page
        return p_Mapping.findForward("ActionEditDomain");
    }
}
