/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999-2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.domain;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;

import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.objectweb.jonas.jmx.J2eeObjectName;
import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.ManagementReprLoader;
import org.objectweb.jonas.management.j2eemanagement.J2EEDomain;
import org.objectweb.jonas.webapp.jonasadmin.JonasBaseAction;
import org.objectweb.jonas.webapp.jonasadmin.common.BeanComparator;
import org.objectweb.jonas.webapp.jonasadmin.common.ObjectNameItem;

/**
 * @author Michel-Ange ANTON
 */

public class EditDomainAction extends JonasBaseAction {

// --------------------------------------------------------- Public Methods

    public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
            , HttpServletRequest p_Request, HttpServletResponse p_Response)
    throws IOException, ServletException {

        try {
            DomainForm oForm = (DomainForm) p_Form;

            // general properties
            String adminServerName = m_WhereAreYou.getAdminJonasServerName();
            String domainName = m_WhereAreYou.getCurrentDomainName();
            ObjectName onCurrentServer = m_WhereAreYou.getCurrentJonasServer();
            ObjectName onDomain = m_WhereAreYou.getCurrentDomain();
            ObjectName onRootDomain = onDomain;

            String selected = p_Request.getParameter("select");
            boolean inCluster = false;
            if (selected != null && !onDomain.toString().equals(selected)) {
                inCluster = true;
                onDomain = ObjectName.getInstance(selected);
                p_Request.setAttribute("clusterName", onDomain.getKeyProperty("clusterName"));
            }
            m_Session.setAttribute("currentOn", onDomain.toString());

            boolean changeManagement = false;
            if (!adminServerName.equals(m_WhereAreYou.getCurrentJonasServerName())) {
                // change management conext from a managed server in the domain
                // to the JonasAdmin server
                ManagementReprLoader.loadServerRepr(adminServerName);
                m_WhereAreYou.refreshServers(p_Request);
                onCurrentServer = m_WhereAreYou.getCurrentJonasServer();
                changeManagement = true;
            }

            // Force the node selected in tree
            m_WhereAreYou.selectNameNode("domain", true);

            if (changeManagement) {
                refreshServerTree(p_Request);
            }

            oForm.setDomainName(domainName);
            oForm.setDescription(getStringAttribute(onDomain, "description"));
            oForm.setCluster(inCluster);
            oForm.setMasterName(adminServerName);
            oForm.setMasterON(J2eeObjectName.J2EEServer(domainName, adminServerName).toString());
            oForm.setMaster(getBooleanAttribute(onDomain, "master"));
            if (inCluster) {
                oForm.setName(onDomain.getKeyProperty("clusterName"));
                String parentName = onDomain.getKeyProperty("parentClusterName");
                oForm.setParentName(parentName);
                String parentON = null;
                if (parentName.equals(domainName)) {
                    parentON = onRootDomain.toString();
                } else {
                    ObjectName parentClusters = J2eeObjectName.J2EEClusters(domainName, parentName);
                    ObjectName parentOn = (ObjectName) JonasManagementRepr.queryNames(parentClusters).iterator().next();
                    parentON = parentOn.toString();
                }
                oForm.setParentON(parentON);
                oForm.setType("Cluster");
            } else {
                oForm.setName(domainName);
                oForm.setType("Domain");
            }

            // Get servers running in this domain (or cluster)
            String[] serverOns = (String[]) JonasManagementRepr.getAttribute(onDomain, "servers");
            String[] serverNames = (String[]) JonasManagementRepr.getAttribute(onDomain, "serverNames");
            ArrayList al = new ArrayList();
            int nbCreatedServers = 0;
            String sOn = null;
            for (int i = 0; i < serverNames.length; i++) {
                String serverName = serverNames[i];
                ObjectName onServer = J2eeObjectName.J2EEServer(domainName, serverName);

                if (!onServer.equals(onCurrentServer) && isServerInServersList(onServer.toString(), serverOns)) {
                    String[] signature = { "java.lang.String" };
                    String[] params = new String[1];
                    params[0] = serverName;
                    String state = (String) JonasManagementRepr.invoke(onDomain, "getServerState", params, signature);
                    if (state.equals(J2EEDomain.STOPPED)) {
                        nbCreatedServers++;
                    }
                    al.add(new ServerItem(onServer, state));
                }
            }
            oForm.setNbCreatedServers(nbCreatedServers);

            Collections.sort(al, new BeanComparator(new String[] {
            "name"}));
            p_Request.setAttribute("listServers", al);

            // Get clusters in thie domain (or cluster)
            ArrayList clustAl = new ArrayList();
            String[] clustersOn = (String[]) JonasManagementRepr.getAttribute(onDomain, "clusters");
            String[] currentClusters = new String[clustersOn.length];
            for (int j = 0; j < clustersOn.length; j++) {
                sOn = clustersOn[j];
                ObjectName onCluster = ObjectName.getInstance(sOn);
                ObjectNameItem item = new ObjectNameItem(onCluster);
                String itemName = onCluster.getKeyProperty("clusterName");
                item.setName(itemName);
                clustAl.add(item);
                currentClusters[j] = itemName;
            }
            Collections.sort(clustAl, new BeanComparator(new String[] {
                "name"}));
            p_Request.setAttribute("listClusters", clustAl);
            m_Session.setAttribute("currentClusters", currentClusters);

        } catch (Throwable t) {
            addGlobalError(t);
            saveErrors(p_Request, m_Errors);
            return (p_Mapping.findForward("Global Error"));
        }
        // Forward to the jsp.
        return (p_Mapping.findForward("Domain"));
    }

    private boolean isServerInServersList(String on, String[] serverOns) {
        boolean result = false;
        for (int i = 0; i < serverOns.length; i++) {
            if (serverOns[i].equals(on)) {
                return true;
            }
        }
        return result;
    }
}
