/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 * Copyright (C) 2006 Distributed Systems Lab.
 * Universidad Polit??cnica de Madrid (Spain)
 * Contact: http://lsd.ls.fi.upm.es/lsd
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 *
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.container;

import java.rmi.NoSuchObjectException;
import java.rmi.Remote;
import java.rmi.RemoteException;

import org.objectweb.carol.cmi.ClusterStub;
import org.objectweb.carol.cmi.CmiOutputStream;
import org.objectweb.carol.cmi.DistributedEquiv;
import org.objectweb.carol.cmi.LowerOrb;
import org.objectweb.carol.cmi.ServerConfigException;
import org.objectweb.carol.cmi.StubConfig;
import org.objectweb.carol.cmi.StubConfigException;
import org.objectweb.carol.cmi.StubData;
import org.objectweb.carol.cmi.configuration.TraceCmi;
import org.objectweb.carol.cmi.ha.ReplicationManager;
import org.objectweb.carol.util.configuration.ConfigurationRepository;

import org.objectweb.jonas_ejb.deployment.api.SessionDesc;

import org.objectweb.jonas.ha.HaUtilsDelegateImpl;

/**
 * This class is the Standard Home for replicated SFSB objects It exists only
 * for beans that have declared a Remote Interface.
 * @author Francisco Perez-Sorrosal (fpsorrosal@no-spam@fi.upm.es)
 * @author Alberto Paz-Jimenez (apaz@no-spam@fi.upm.es)
 * @author Benoit Pelletier
 */
public abstract class JRepStatefulHome extends JSessionHome {

    private static ReplicationManager repMgr = null;

    public JRepStatefulHome(SessionDesc dd, JSessionFactory bf) throws RemoteException {
        super(dd, bf);
        TraceCmi.debugCmiHA("JRepStatefulHome");
        // Get a reference to the replication manager from the HA service
        if (repMgr == null) repMgr = HaUtilsDelegateImpl.getReplicationManager();
    }

    /* ******************REPLICATION CODE STARTS HERE*********************** */

    /**
     * Get the remote object as a stub object for HA remote object LB
     * @param remote remote object
     * @return get remote object
     * @throws RemoteException
     * @throws ServerConfigException
     * @throws NoSuchObjectException if the clustered object is not found in the
     *         CMI registry
     */
    public Remote getHARemoteStub(Remote remote) throws RemoteException {
        String protocol = ConfigurationRepository.getCurrentConfiguration().getProtocol().getName();
        if (protocol.equals("cmi")) {
            StubData sd;
            ClusterStub cs;
            try {
                Remote remObject = LowerOrb.getPRODelegate().toStub(remote);
                byte[] ser = CmiOutputStream.serialize(dd.getJndiName(), remObject);
                // Need to obtain an ObjectId from DistributedEquivSystem.
                sd = new StubData(DistributedEquiv.getOidFactory().getId(), ser, 100);
                sd.setKey(dd.getJndiName());
                cs = StubConfig.instanciateClusterStub(remote.getClass(), sd);
            } catch (RemoteException e) {
                e.printStackTrace();
                throw e;
            } catch (ServerConfigException e) {
                e.printStackTrace();
                throw new StubConfigException(e.getMessage());
            }
            return cs;
        }

        return remote;
    }

    /* **********************END OF REPLICATION CODE************************ */

}